import jgtpy
from jgtpy import jgtfxcommon
import argparse

from jgtpy import JGTPDS as pds

import pandas as pd

def parse_args():
    parser = argparse.ArgumentParser(description='Process command parameters.')
    #jgtfxcommon.add_main_arguments(parser)
    jgtfxcommon.add_instrument_timeframe_arguments(parser)
    jgtfxcommon.add_date_arguments(parser)
    jgtfxcommon.add_max_bars_arguments(parser)
    jgtfxcommon.add_output_argument(parser)
    args = parser.parse_args()
    return args


def main():
    args = parse_args()
    instrument = args.i
    timeframe = args.timeframe
    quotes_count = args.quotescount
    date_from = args.datefrom
    date_to = args.dateto
    output = args.output

    try:
        print("Getting for : " + instrument + "_" + timeframe)
        p=pds.getPH(instrument,timeframe)
        print(p)
    except Exception as e:
        jgtfxcommon.print_exception(e)

    try:
        jgtpy.off()
    except Exception as e:
        jgtfxcommon.print_exception(e)

# if __name__ == "__main__":
#     main()

# print("")
# #input("Done! Press enter key to exit\n")



# Copyright 2019 Gehtsoft USA LLC

# Licensed under the license derived from the Apache License, Version 2.0 (the "License"); 
# you may not use this file except in compliance with the License.

# You may obtain a copy of the License at

# http://fxcodebase.com/licenses/open-source/license.html

# Unless required by applicable law or agreed to in writing, software 
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


from jgtcommon.common import add_main_arguments, add_instrument_timeframe_arguments, \
    add_candle_open_price_mode_argument, add_direction_rate_lots_arguments, add_account_arguments, \
    valid_datetime, add_date_arguments, add_report_date_arguments, add_max_bars_arguments, add_bars_arguments, \
    print_exception, diff_month, add_nb2retrieve_arguments,add_out,add_out_pov,get_opov_filenamed,add_local_arguments



# #import fxcmpy and check the imported version
# import forexconnect as fx
# import datetime as dt
# import pandas as pd
# #fxcmpy.__version__
# import jgtpy.JGTConfig as jgtc


# con=None

# def create_con():
#   return fx.fxcmpy(access_token = token, log_level='error',log_file=None)
# #config_file='fxcm.cfg')

# def create_con_cfg():
#   return fx.fxcmpy(config_file='fxcm.cfg', log_level='error',log_file=None)



# def connect(quiet=True):
#   global con
#   try:
#     if not con or not con.connection_status == 'established':
#       con = create_con()
#       if not quiet:
#         print('Connected')
#     else:
#       if not quiet:
#         print('ALready Connected')
#   except ConnectionError:
#     print('Connection Error, retrying')
#     con = create_con()
#   except TimeoutError                              :
#     print('ohoh, Timeout, retrying...')
#     try:
#       con = None
#       con = create_con()
#     except ConnectionError:
#       print('ohoh AGAIN, Connection error, retrying...FUCK OFF')
#       pass
#     except TimeoutError                              :
#       print('ohoh AGAIN, Timeout, retrying...FUCK OFF')
#       pass
#   return con
#   #print(con.connection_status)



# def disconnect(quiet=False):
#   global con
#   if con and con.connection_status == 'established' :
#     con.close()
#     if not quiet:
#       print('Disconnected')
#   else:
#     if not quiet:
#       print('Not connected')
#   con= None
#   return con
  
# def status():
#   global con
#   if con :
#     print(con.connection_status)
#   else:
#     print('Not connected')




# %%

from . import JGTPDS as pds
from . import JGTIDS as ids

import pandas as pd
import os

def startSession():
  pds.connect()
  
def stopSession():
  pds.disconnect()
# %%

def createFromPDSFile(_instrument,_timeframe,quiet=True):
  """Create CDS (Chaos Data Service) with Fresh Data

  Args:
      _instrument (str): symbol
      _timeframe (str): TF
      _path (str): path to file

  Returns:
      pandas.DataFrame: CDS DataFrame
  """
  df=pds.getPH_from_filestore(_instrument,_timeframe)
  if not quiet:
    print(df)
  dfi=createFromDF(df)
  return dfi

def createFromDF(df, quiet=True):
  """
  Creates a new DataFrame with indicators, signals, and cleansed columns added based on the input DataFrame.

  Args:
    df (pandas.DataFrame): The input DataFrame to add indicators, signals, and cleansed columns to.
    quiet (bool, optional): Whether to suppress console output during processing. Defaults to True.

  Returns:
    pandas.DataFrame: The new DataFrame with indicators, signals, and cleansed columns added.
  """
  dfi=ids.ids_add_indicators(df,quiet=quiet)
  dfi=ids.cds_add_signals_to_indicators(dfi,quiet=quiet)
  dfi=ids.jgti_add_zlc_plus_other_AO_signal(dfi,quiet=quiet)
  dfi=ids.pds_cleanse_original_columns(dfi,quiet=quiet)
  dfi =ids.__ids_cleanse_ao_peak_secondary_columns(dfi,quiet=True)
  return dfi


def create(instrument,timeframe,nb2retrieve=335,stayConnected=False,quiet=True):
  """Create CDS (Chaos Data Service) with Fresh Data

  Args:
      instrument (str): symbol
      timeframe (str): TF
      nb2retrieve (int, optional): nb bar to retrieve. Defaults to 335.
      stayConnected (bool, optional): Leave Forexconnect connected. Defaults to False.
      quiet (bool,optional): Output quiet

  Returns:
      pandas.DataFrame: CDS DataFrame
  """
  pds.stayConnected=stayConnected
  df=pds.getPH(instrument,timeframe,nb2retrieve,with_index=False,quiet=quiet)
  dfi=createFromDF(df,quiet=quiet)
  return dfi
  

#createByRange
def createByRange(instrument,timeframe,start,end,stayConnected=False,quiet=True):
  """Create CDS with Fresh Data from a range

  Args:
      instrument (str): symbol
      timeframe (str): TF
      start (date): start date
      end (date): end date
      stayConnected (bool, optional): Leave FXCMPY connected. Defaults to False.
      quiet (bool,optional): Output quiet

  Returns:
      pandas.DataFrame: CDS DataFrame
  """
  pds.stayConnected=stayConnected
  df=pds.getPHByRange(instrument,timeframe,start,end,with_index=False,quiet=quiet)
  dfi=createFromDF(df,quiet=quiet)
  return dfi





columns_to_remove = ['aofvalue', 'aofhighao', 'aoflowao', 'aofhigh', 'aoflow', 'aocolor', 'accolor', 'fdbbhigh', 'fdbblow', 'fdbshigh', 'fdbslow']
def create_and_clean_data_from_file_df(instrument, timeframe):
    # Create DataFrame from PDS file
    c = createFromPDSFile(instrument, timeframe)

    # Remove specified columns if provided
    if columns_to_remove:
        c = c.drop(columns=columns_to_remove, errors='ignore')

    # Set 'Date' as the index
    c.set_index('Date', inplace=True)

    return c


def get_data_path():
    data_path = os.environ.get('JGTPY_DATA', './data')
    data_path = os.path.join(data_path, 'cds')
    return data_path
  
def _save_cds_data_to_file(df, instrument, timeframe):
    # Define the file path based on the environment variable or local path
    data_path = get_data_path()
    fpath = pds.mk_fullpath(instrument, timeframe, 'csv', data_path)

    # Save DataFrame to CSV
    df.to_csv(fpath)
    return fpath

def createFromFile_and_clean_and_save_data(instrument, timeframe):
    # Create DataFrame from PDS file
    c =create_and_clean_data_from_file_df(instrument, timeframe)
    _save_cds_data_to_file(c, instrument, timeframe)


    return c









def getSubscribed():
  return pds.getSubscribed()

def getActiveSymbols():
  AppSuiteConfigRootPath = os.getenv('AppSuiteConfigRootPath')
  fn='activesymbol.txt'
  fpath=os.path.join(AppSuiteConfigRootPath,fn)
  with open(fpath) as f:
    first_line = f.readline()
    print(first_line)

# %%
def getLast(_df):
  return _df.iloc[-1]

def getPresentBar(_df):
  r= _df#['High','Low',indicator_AO_awesomeOscillator_column_name,signalCode_fractalDivergentBar_column_name,indicator_AC_accelerationDeceleration_column_name]
  return r.iloc[-1:]

def getPresentBarAsList(_df):
  _paf =_df.iloc[-1:]
  _pa = _paf.to_dict(orient='list')
  _dtctx=str(_paf.index.values[0])
  _pa['Date'] = _dtctx
  return _pa


def getLastCompletedBarAsList(_df):
  _paf =_df.iloc[-2:-1]
  _pa = _paf.to_dict(orient='list')
  _dtctx=str(_paf.index.values[0])
  _pa['Date'] = _dtctx
  return _pa





def checkFDB(_instrument,_timeframe):
  _df=create(_instrument)
  pa = getPresentBarAsList(_df)
  isfdb = pa[signalCode_fractalDivergentBar_column_name][0]  != 0.0
  fdb = pa[signalCode_fractalDivergentBar_column_name]
  dtctx = pa['Date']
  if isfdb:
    print(_instrument + "_" + _timeframe + " : We Have a Signal : " + dtctx)
    return True
  else:
    print(_instrument + "_" + _timeframe +  " : No signal now : " + dtctx)
    return False



def print_quiet(quiet,content):
    if not quiet:
        print(content)




nonTradingZoneColor = 'gray'

sellingZoneColor = 'red'
buyingZoneColor = 'green'


indicator_currentDegree_alligator_jaw_column_name = 'jaw' # 13 periods moving average, shifted 8 bars into the future
indicator_currentDegree_alligator_teeth_column_name = 'teeth' # 8 periods moving average, shifted 5 bars into the future
indicator_currentDegree_alligator_lips_column_name = 'lips' # 5 periods moving average, shifted 3 bars into the future
indicator_sixDegreeLarger_alligator_jaw_column_name = 'bjaw' # 89 periods moving average, shifted 55 bars into the future
indicator_sixDegreeLarger_alligator_teeth_column_name = 'bteeth' # 55 periods moving average, shifted 34 bars into the future
indicator_sixDegreeLarger_alligator_lips_column_name = 'blips' # 34 periods moving average, shifted 21 bars into the future

indicator_AO_awesomeOscillator_column_name = 'ao' # AO measure energy of momentum
indicator_AC_accelerationDeceleration_column_name = 'ac' # AC measure speed of momentum
indicator_AO_aboveZero_column_name = 'aoaz'
indicator_AO_bellowZero_column_name = 'aobz'
indicator_zeroLineCross_column_name = 'zlc'

indicator_gatorOscillator_low_column_name = 'gl' # Gator Oscillator low
indicator_gatorOscillator_high_column_name = 'gh' # Gator Oscillator high


indicator_mfi_marketFacilitationIndex_column_name = 'mfi' # MFI measure market facilitation index
    
#Various fractal degrees
indicator_fractal_high_degree2_column_name="fh" # Fractal High of degree 2
indicator_fractal_low_degree2_column_name="fl" # Fractal Low of degree 2
indicator_fractal_high_degree3_column_name="fh3" # Fractal High of degree 3
indicator_fractal_low_degree3_column_name="fl3" # Fractal Low of degree 3
indicator_fractal_high_degree5_column_name="fh5" # Fractal High of degree 5
indicator_fractal_low_degree5_column_name="fl5" # Fractal Low of degree 5
indicator_fractal_high_degree8_column_name="fh8" # Fractal High of degree 8
indicator_fractal_low_degree8_column_name="fl8" # Fractal Low of degree 8
indicator_fractal_high_degree13_column_name="fh13" # Fractal High of degree 13
indicator_fractal_low_degree13_column_name="fl13" # Fractal Low of degree 13
indicator_fractal_high_degree21_column_name="fh21" # Fractal High of degree 21
indicator_fractal_low_degree21_column_name="fl21" # Fractal Low of degree 21
indicator_fractal_high_degree34_column_name="fh34" # Fractal High of degree 34
indicator_fractal_low_degree34_column_name="fl34" # Fractal Low of degree 34
indicator_fractal_high_degree55_column_name="fh55" # Fractal High of degree 55
indicator_fractal_low_degree55_column_name="fl55" # Fractal Low of degree 55
indicator_fractal_high_degree89_column_name="fh89" # Fractal High of degree 89
indicator_fractal_low_degree89_column_name="fl89" # Fractal Low of degree 89
indicator_ao_fractalPeakOfMomentum_column_name = 'aof'
indicator_ao_fractalPeakValue_column_name = 'aofvalue'
# %%
#@title SIGNAL's Data Frame Columns naming

# fractal divergent bar signals (or BDB)
signalCode_fractalDivergentBar_column_name = 'fdb' # Fractal Divergent Bar Code (contains the signal value either buy, sell or nothing)
signalSell_fractalDivergentBar_column_name = 'fdbs' # Fractal Divergent Bar Sell
signalBuy_fractalDivergentBar_column_name = 'fdbb' # Fractal Divergent Bar Buy

#ac
signalSell_AC_deceleration_column_name = 'acs'
signalBuy_AC_acceleration_column_name = 'acb'

#Fractal signal
signalSell_fractal_column_name = 'fs'
signalBuy_fractal_column_name = 'fb'

#Zero Line Cross Signal
signalBuy_zeroLineCrossing_column_name = 'zlcb'
signalSell_zeroLineCrossing_column_name = 'zlcs'
signal_zcol_column_name = 'zcol' # NOT SURE its a signal

signalSell_zoneSignal_column_name = 'sz'
signalBuy_zoneSinal_column_name = 'bz'

# Saucer signal
signalSell_saucer_column_name = 'ss'
signalBuy_saucer_column_name = 'sb'



# JGT PDS
#
# Copyright 20.1.4022 Jean Guillaume Isabelle
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
.. moduleauthor:: Jean Guillaume Isabelle <jgi@jgwill.com>
"""

# from jgtfxcommon.BatchOrderMonitor import BatchOrderMonitor
# from jgtfxcommon.OrderMonitor import OrderMonitor
# from jgtfxcommon.OrderMonitorNetting import OrderMonitorNetting
# from jgtfxcommon.TableListenerContainer import TableListenerContainer
# from jgtfxcommon.common import add_main_arguments, add_instrument_timeframe_arguments, \
#     add_candle_open_price_mode_argument, add_direction_rate_lots_arguments, add_account_arguments, \
#     valid_datetime, add_date_arguments, add_report_date_arguments, add_max_bars_arguments, add_bars_arguments, \
#     print_exception, session_status_changed, diff_month, convert_timeframe_to_seconds


#from jgtpy.common_samples import common_samples
from .jgtetl import svc_offset_dt_by_tf as etl_offset_dt_by_tf,offsetdt as etl_offsetdt
from .JGTCore import __version__ #,json2dict,jsonfile2prop,json2prop,jsonfile2dict,d2p,fixdtindf,offsetdt
#from .JGTConfig import getenv,setreal,setdemo,env

from .JGTPDS import getPH as get_price, stayConnectedSetter as set_stay_connected, disconnect,connect as on,disconnect as off, status as connection_status,  getPH2file as get_price_to_file, getPHByRange as get_price_range 
#mk_fn,mk_fullpath,getSubscribed,getPH,getPHByRange,tryConnect
from .JGTCDS import create as createCDS,createByRange,createFromDF,startSession,stopSession,getLast,getPresentBar,getPresentBarAsList,getLastCompletedBarAsList,createFromDF
# from jgtpy.JGTADS import ads_chart_pto,retrieve_n_chart as ads_retrieve_n_chart

# from .jgtcli import main as __main__
#from jgtpy.jgtconstants import *



#@title Plotly

import pandas as pd

import kaleido
import plotly

import jgtpy.JGTCDS as cds
import jgtpy.JGTConfig as jgtc

from matplotlib import pyplot as plt

import plotly.graph_objects as go
import plotly.subplots as sp


cdtformat="%Y-%m-%d"




from .jgtconstants import (
  sellingZoneColor,
  buyingZoneColor,
)

# %%
#@title INDICATOR's Data Frame Columns naming
# Import statements for jgtconstants.py variables


from .jgtconstants import (
    indicator_currentDegree_alligator_jaw_column_name,
    indicator_currentDegree_alligator_teeth_column_name,
    indicator_currentDegree_alligator_lips_column_name,
    indicator_sixDegreeLarger_alligator_jaw_column_name,
    indicator_sixDegreeLarger_alligator_teeth_column_name,
    indicator_sixDegreeLarger_alligator_lips_column_name,
    indicator_AO_awesomeOscillator_column_name,
    indicator_zeroLineCross_column_name,
    indicator_ao_fractalPeakOfMomentum_column_name,
    indicator_ao_fractalPeakValue_column_name,
    indicator_AO_aboveZero_column_name,
    indicator_AO_bellowZero_column_name,
    indicator_AC_accelerationDeceleration_column_name,
    indicator_gatorOscillator_low_column_name,
    indicator_gatorOscillator_high_column_name,
    indicator_mfi_marketFacilitationIndex_column_name,
    indicator_fractal_high_degree2_column_name,
    indicator_fractal_low_degree2_column_name,
    indicator_fractal_high_degree3_column_name,
    indicator_fractal_low_degree3_column_name,
    indicator_fractal_high_degree5_column_name,
    indicator_fractal_low_degree5_column_name,
    indicator_fractal_high_degree8_column_name,
    indicator_fractal_low_degree8_column_name,
    indicator_fractal_high_degree13_column_name,
    indicator_fractal_low_degree13_column_name,
    indicator_fractal_high_degree21_column_name,
    indicator_fractal_low_degree21_column_name,
    indicator_fractal_high_degree34_column_name,
    indicator_fractal_low_degree34_column_name,
    indicator_fractal_high_degree55_column_name,
    indicator_fractal_low_degree55_column_name,
    indicator_fractal_high_degree89_column_name,
    indicator_fractal_low_degree89_column_name,
)


# %%
#@title SIGNAL's Data Frame Columns naming
# Import statements for jgtconstants.py variables

from .jgtconstants import (
    signalCode_fractalDivergentBar_column_name,
    signalSell_fractalDivergentBar_column_name,
    signalBuy_fractalDivergentBar_column_name,
    signalSell_fractal_column_name,
    signalBuy_fractal_column_name,
    signal_zcol_column_name,
    signalSell_zoneSignal_column_name,
    signalBuy_zoneSinal_column_name,
    signalBuy_zeroLineCrossing_column_name,
    signalSell_zeroLineCrossing_column_name,
    signalSell_AC_deceleration_column_name,
    signalBuy_AC_acceleration_column_name,
    signalSell_saucer_column_name,
    signalBuy_saucer_column_name,
)

# if timeframe=='m5':
# 	cdtformat='%H:%M'
 


#@title JGT Plotting Functions
def jgtplt_createSubplots(rows=4,row_heights=[50,15,15],cols=1,shared_xaxes=True):
  return sp.make_subplots(rows=rows, cols=cols,row_heights=row_heights,shared_xaxes=shared_xaxes)

def addline2main(_fig,_data,xdata,name,color,width=1,opacity=0.5,row=1,col=1):
  _fig.add_trace(go.Scatter(
    x=xdata, y=_data, opacity=opacity,
    line=dict(color=color, width=width), name=name),
    row=row, col=col)
  return _fig
 
 
 #@title DEF Charting Function PTO 221025

#@title RESTORE RAnge break

def _ads_get_dt_break(dfsrc,_timeframe='',_cdtformat="%Y-%m-%d"):
  if _timeframe=='m5' or _timeframe == 'm1' or _timeframe =='mi1':
    _cdtformat='%H:%M'
  _dt_all = pd.date_range(start=dfsrc.index[0],end=dfsrc.index[-1])
  # retrieve the dates that ARE in the original datset
  _dt_obs = [d.strftime(_cdtformat) for d in pd.to_datetime(dfsrc.index)]
  # define dates with missing values
  _dt_breaks = [d for d in _dt_all.strftime("%Y-%m-%d").tolist() if not d in _dt_obs]
  return _dt_breaks


 
#@title Coloring the AO  
# df['aocolor'] = 'rgba(255, 0, 0, 1.0)'
# if df[[df[indicator_AO_awesomeOscillator_column_name]] > df[df['pao0']]:
#   df['aocolor'] = 'rgba(0, 255, 0, 1.0)'
# df.tail()

def retrieve_n_chart_pano(_instrument,_nb2retrieve=335,_cw =1050,_ch =1000,useDTBreaks=False):
  charts={}
  for _timeframe in jgtc.contextTimeframes:
    print('ADS is retrieving for: ' + _instrument + '_'+ _timeframe)
    _df=cds.create(_instrument,_timeframe,_nb2retrieve,True)
    f4=ads_chart_pto(_df,_timeframe,_cw,_ch,useDTBreaks)
    charts[_timeframe]= f4
  return charts

def retrieve_n_chart(_instrument,_timeframe,_nb2retrieve=335,_cw =1050,_ch =1000,useDTBreaks=False):
  print('ADS is retrieving for: ' + _instrument + '_'+ _timeframe)
  _df=cds.create(_instrument,_timeframe,_nb2retrieve)
  f4=ads_chart_pto(_df,_timeframe,_cw,_ch,useDTBreaks)
  return f4



def ads_chart_pto(dfsrc,_timeframe,_cw =1050,_ch =1000,useDTBreaks=False)-> plt.Figure:
  fdball =  dfsrc[dfsrc[signalCode_fractalDivergentBar_column_name]!=0 ] 
  fdbs = dfsrc[dfsrc[signalSell_fractalDivergentBar_column_name]==True]
  fdbb = dfsrc[dfsrc[signalBuy_fractalDivergentBar_column_name]==True]
  aocolor=dfsrc['aocolor'].values.tolist()
  
  zcolor=dfsrc[signal_zcol_column_name].values.tolist()
  #print(aocolor)
  #@title Arrays for Charting
  _dt_breaks=_ads_get_dt_break(dfsrc,_timeframe)
  _fdball=dfsrc[dfsrc[signalCode_fractalDivergentBar_column_name] != 0 ]
  _open_data = dfsrc['Open']
  _close_data = dfsrc['Close']
  _high_data = dfsrc['High']
  _low_data = dfsrc['Low']
  _ao_data = dfsrc[indicator_AO_awesomeOscillator_column_name]
  _ac_data = dfsrc[indicator_AC_accelerationDeceleration_column_name]
  #dates = _df['Date']
  _dates =dfsrc.index #dates are used as index, hope its ok
  _dates_fdball = fdball.index
  _dates_fdbs = fdbs.index
  _dates_fdbb = fdbb.index
  #print(dates)
  _fdbs_data = dfsrc[signalSell_fractalDivergentBar_column_name]
  _fdbb_data = dfsrc[signalBuy_fractalDivergentBar_column_name]
  _fs_data = dfsrc[signalSell_fractal_column_name]
  _fb_data = dfsrc[signalBuy_fractal_column_name]
  
  _zlc_data = dfsrc[indicator_zeroLineCross_column_name]
  _saucersell_data=dfsrc[signalSell_saucer_column_name]
  _saucerbuy_data=dfsrc[signalBuy_saucer_column_name]
  _zcol_data = dfsrc[signal_zcol_column_name]
  # fig = go.Figure(data=[go.Ohlc(x=dates,
  #                           open=open_data, high=high_data,
  #                           low=low_data, close=close_data)])
  # fig.show()
  # Plot OHLC on 1st subplot (using the codes from before)
  _f4 = sp.make_subplots(rows=3, cols=1,row_heights=[75,20,15],shared_xaxes=True)
                          #, specs=[[{'l': 0},{'r': 0}], [{'t': 0}, {'b': 0}]], shared_xaxes=True)
  mainrow=1
  aorow=2
  acrow=3
  aopeakrow=2
  oh=go.Ohlc(  
      x=_dates,
      open=_open_data,
      high=_high_data,
      low=_low_data,
      close=_close_data, 
      #color=df[signal_zcol_column_name],
      showlegend=False)
            # increasing_line_color=df[signal_zcol_column_name], decreasing_line_color=df[signal_zcol_column_name],

  _f4.add_trace(oh,row=mainrow,col=1)

  # Try to add some stuff when FDB
  #fig4.add_trace(go.Annotation('X',x=dates,
  #                            y=fdbb_data,row=1,col=1))
  gatorlinewidth=2
  # add Alligator
  _f4=addline2main(_f4,dfsrc[indicator_currentDegree_alligator_jaw_column_name],_dates,'Jaw','blue',gatorlinewidth,0.6,mainrow,1)
  _f4=addline2main(_f4,dfsrc[indicator_currentDegree_alligator_teeth_column_name],_dates,'Teeth',sellingZoneColor,gatorlinewidth,0.6,mainrow,1)
  _f4=addline2main(_f4,dfsrc[indicator_currentDegree_alligator_lips_column_name],_dates,'Lips',buyingZoneColor,gatorlinewidth,0.6,mainrow,1)


  # add Raptor (big Alligator
  _f4=addline2main(_f4,dfsrc[indicator_sixDegreeLarger_alligator_jaw_column_name],_dates,'BJaw','cyan',3,0.4,mainrow)
  _f4=addline2main(_f4,dfsrc[indicator_sixDegreeLarger_alligator_teeth_column_name],_dates,'BTeeth','magenta',3,0.4,mainrow)
  _f4=addline2main(_f4,dfsrc[indicator_sixDegreeLarger_alligator_lips_column_name],_dates,'BLips','yellow',3,0.4,mainrow)

  # fdbplot = go.Figure(data=[go.Scatter(
  #      x = df.index,
  #      y = df[signalBuy_fractalDivergentBar_column_name],
  #      mode = 'markers',)
  #  ])

  #fdbbpx= px.scatter(df,x='Date',y=signalBuy_fractalDivergentBar_column_name,color='aocolor')
  # fdbbpx= px.scatter(x=dates,y=fdbb_data)
  # fig4.add_trace(fdbbpx,row=1, col=1)
  #fig4.add_trace(go.Scatter(x=dates,y=fdbb_data),row=1, col=1)

  _f4.add_trace(go.Bar( 
                      x=_dates,
                      y=_ao_data,    
                      marker=dict(color=dfsrc['aocolor'].values.tolist(),
                      line=dict(color=dfsrc['aocolor'].values.tolist(), width=1)),
                      name='AO'), row=aorow, col=1)



  _f4.add_trace(go.Bar( 
                      x=_dates,
                      y=_ac_data,    
                      marker=dict(color=dfsrc['accolor'].values.tolist(),
                      line=dict(color=dfsrc['accolor'].values.tolist(), width=1)),
                      name='AC'), row=acrow, col=1)

  ############################## Ref to draw a Red or Green Zone
  # fig.add_shape(type="rect",
  #     x0=3, y0=1, x1=6, y1=2,
  #     line=dict(
  #         color="RoyalBlue",
  #         width=2,
  #     ),
  #     fillcolor="LightSkyBlue",
  # )

  ###################################### --@STCIssue How to ADd x0 and x1 ?
  # f4.add_shape(type="rect",
  #     x0=dtStart,y0.1.481987,x1=dtEnd, y1=1.3780,
  #     line=dict(
  #         color="RoyalBlue",
  #         width=2,
  #     ),
  #     fillcolor="LightSkyBlue",
  # )

  # ALL FDB PLOTTING using A SCATTER 
  _fdbplot =  go.Scatter(
      x = _dates_fdball,
      y = _fdball['High'],
      mode = 'markers',)

  _f4.add_trace(_fdbplot,row=1, col=1)
  # update layout by changing the plot size, hiding legends & rangeslider, and removing gaps between dates
  _f4.update_layout(height=_ch, width=_cw, 
                    showlegend=False, 
                    xaxis_rangeslider_visible=False)
  if useDTBreaks:
    _f4.update_xaxes(rangebreaks=[dict(values=_dt_breaks)])
  return _f4


# A 2x6 chart

def pano_chart_pto221103(charts,_nbrows=3):
  panocharts = sp.make_subplots(rows=_nbrows, 
                                cols=1,
                                row_heights=[110.1.4,110],
                                shared_xaxes=True)
                                #column_widths=[75,75]
  cl=1
  r=1
  for c in charts:
    panocharts.add_trace(charts[c],row=r,col=cl)
    r=r+1
    # cl=cl+1
    # if cl==3:
    #   cl=1
  return panocharts
  







def update(_f4,_ch,_cw,showlegent=False,xaxis_rangeslider_visible=False):
  _f4.update_layout(height=_ch, width=_cw, 
                    showlegend=False, 
                    xaxis_rangeslider_visible=False)


#@title PD Columen Cleanup Functions

def jgtpd_drop_col_by_name(dfsrc,colname,axis = 1,quiet=False):
  """Drop Column in DF by Name

  Args:
        dfsrc (DataFrame source)
        ctxcolname (column name from)
        axis (  axis)
        quiet (quiet output)
        
  Returns:
    Clean DataFrame 
  """
  if colname in dfsrc.columns:
    return dfsrc.drop(dfsrc.loc[:, colname:colname].columns,axis = axis)
  else:
    if not quiet:
      print('Col:' + colname + ' was not there')
    return dfsrc

def _ids_cleanse_ao_peak_secondary_columns(dfsrc,quiet=False):
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'p0',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'p1',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'p2',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'p3',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'p4',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'p5',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'p6',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'p7',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'p8',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'p9',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'p10',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'p11',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'p12',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'p13',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'p14',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'p15',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'p16',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'p17',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'p18',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'p19',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'p20',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'p21',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'p22',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'p23',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'p24',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'p25',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'p26',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'p27',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'p28',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'p29',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'p30',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'n0',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'n1',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'n2',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'n3',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'n4',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'n5',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'n6',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'n7',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'n8',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'n9',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'n10',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'n11',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'n12',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'n13',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'n14',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'n15',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'n16',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'n17',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'n18',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'n19',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'n20',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'n21',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'n22',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'n23',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'n24',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'n25',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'n26',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'n27',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'n28',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'n29',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'n30',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'pao0',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'pao1',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'pao2',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'pao3',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'pao4',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'pao5',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'pao6',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'pao7',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'pao8',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'pao9',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'pao10',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'pao11',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'pao12',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'pao13',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'pao14',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'pao15',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'pao16',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'pao17',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'pao18',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'pao19',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'pao20',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'pao21',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'pao22',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'pao23',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'pao24',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'pao25',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'pao26',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'pao27',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'pao28',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'pao29',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'pao30',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'nao0',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'nao1',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'nao2',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'nao3',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'nao4',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'nao5',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'nao6',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'nao7',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'nao8',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'nao9',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'nao10',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'nao11',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'nao12',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'nao13',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'nao14',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'nao15',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'nao16',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'nao17',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'nao18',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'nao19',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'nao20',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'nao21',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'nao22',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'nao23',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'nao24',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'nao25',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'nao26',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'nao27',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'nao28',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'nao29',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'nao30',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'pac0',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'pac',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'pac1',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'pac2',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'pac3',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'pac4',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'pac5',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'pac6',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'pac7',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'pac8',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'pac9',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'nac',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'nac0',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'nac1',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'nac2',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'nac3',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'nac4',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'nac5',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'nac6',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'nac7',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'nac8',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'nac9',1,quiet) 
  return dfsrc

def pds_cleanse_original_columns(dfsrc,quiet=True):
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'AskHigh',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'BidHigh',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'AskLow',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'BidLow',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'AskClose',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'BidClose',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'BidOpen',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'AskOpen',1,quiet)
  return dfsrc

def pds_cleanse_extra_columns(dfsrc,quiet=True):
  dfsrc=pds_cleanse_original_columns(dfsrc,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'LowisBellowJaw',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'HighisAboveJaw',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'LowisBellowTeeth',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'HighisAboveTeeth',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'HighisAboveLips',1,quiet)
  dfsrc=jgtpd_drop_col_by_name(dfsrc,'LowisBellowLips',1,quiet)
  dfsrc=_ids_cleanse_ao_peak_secondary_columns(dfsrc,quiet)
  if not quiet:
    print("Columns cleanup was executed")
  return dfsrc